package cn.com.ty.lift.enterprise.push;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.com.ty.lift.common.model.PushUserInfo;
import cn.com.ty.lift.enterprise.push.mapper.PushUserMapper;
import cn.com.xwy.boot.service.impl.BaseServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.List;
import java.util.Objects;

/**
 * @author huangyuan
 * @date 2020/4/17
 * @description
 */
@Service
public class PushUserService extends BaseServiceImpl<PushUserMapper, PushUserInfo> {

    /**
     * @return 当前用户id
     * @description 获取当前用户id
     * @date 2020/4/17 10:06 上午
     */
    private Long getCurrentUserId() {
        return getAttributeFromSession(ApiConstants.CURRENT_USER_ID);
    }


    /**
     * @return 当前公司id
     * @description 获取当前公司id
     * @date 2020/4/17 10:09 上午
     */
    private Long getCurrentCompanyId() {
        return getAttributeFromSession(ApiConstants.CURRENT_COMPANY_ID);
    }

    /**
     * @param attributeName 属性名
     * @return 属性值
     * @description 从session中获取
     * @date 2020/4/17 10:09 上午
     */
    private Long getAttributeFromSession(String attributeName) {
        HttpSession session = getHttpSession();
        if (session != null) {
            return (Long) session.getAttribute(attributeName);
        }
        return null;
    }

    /**
     * @return session
     * @description 获取session
     * @date 2020/4/17 10:06 上午
     */
    private HttpSession getHttpSession() {
        HttpServletRequest request = ((ServletRequestAttributes) Objects.requireNonNull(
                RequestContextHolder.getRequestAttributes())).getRequest();
        return request.getSession();
    }

    public PushUserInfo listByUserId(Long mtCompanyId, Long userId){
        return baseMapper.listByUserId(mtCompanyId, userId);
    }

    public List<PushUserInfo> listByUserIds(Long mtCompanyId, List<Long> userIds){
        return baseMapper.listByUserIds(mtCompanyId, userIds);
    }

    public List<PushUserInfo> listByRoleCodes(Long mtCompanyId, List<String> roleCodes){
        return baseMapper.listByRoleCodes(mtCompanyId, roleCodes);
    }

    /**
     * 查询公司下总经理
     * @param mtCompanyId
     * @return
     */
    public List<PushUserInfo> listHighDirector(Long mtCompanyId){
        return baseMapper.listByRoleCode(mtCompanyId, CommonEnum.DefaultRole.HIGH_DIRECTOR.getCode());
    }
    /**
     * 查询公司下管理员
     * @param mtCompanyId
     * @return
     */
    public List<PushUserInfo> listEnterpriseAdmin(Long mtCompanyId){
        return baseMapper.listByRoleCode(mtCompanyId, CommonEnum.DefaultRole.ENTERPRISE_ADMIN.getCode());
    }

}
